'use strict';

var react = require('react');
var reactRedux = require('react-redux');

var createId = function createId() {
  return '_' + Math.random().toString(36).substr(2, 9);
};

var listeners = {};
var addActionListener = function addActionListener(newListener) {
  var listenerId = createId();
  listeners[listenerId] = newListener;
  return function () {
    return delete listeners[listenerId];
  };
};

var ConfigMiddleware = {
  initialized: false,
  suffixes: {
    request: 'REQUESTED',
    success: 'SUCCEEDED',
    failure: 'FAILED',
    cancel: undefined
  }
};

var isAsyncAction = function isAsyncAction(config, action) {
  return action.type.endsWith(config.request) || action.type.endsWith(config.success) || action.type.endsWith(config.failure);
};

var createDispatchAsyncMiddleware = function createDispatchAsyncMiddleware(config) {
  return function () {
    return function (next) {
      return function (action) {
        try {
          if (config && !ConfigMiddleware.initialized) {
            ConfigMiddleware.suffixes = config;
            ConfigMiddleware.initialized = true;
          }

          if (isAsyncAction(ConfigMiddleware.suffixes, action) && Object.keys(listeners).length > 0) {
            for (var _i = 0, _Object$values = Object.values(listeners); _i < _Object$values.length; _i++) {
              var listener = _Object$values[_i];
              listener(action);
            }
          }
        } catch (error) {
          console.error(error);
        }

        return next(action);
      };
    };
  };
};

function dispatchAsync(dispatch, action) {
  return new Promise(function (resolve) {
    var actionNameBase = action.type.replace("_" + ConfigMiddleware.suffixes.request, '');
    var unsubscribe = addActionListener(function (resultAction) {
      if (resultAction.type === actionNameBase + "_" + ConfigMiddleware.suffixes.success) {
        resolve({
          success: true,
          result: resultAction.payload
        });
        unsubscribe();
      } else if (resultAction.type === actionNameBase + "_" + ConfigMiddleware.suffixes.failure) {
        var error = resultAction.payload instanceof Error ? resultAction.payload : new Error("Action failure: " + actionNameBase);
        resolve({
          success: false,
          error: error
        });
        unsubscribe();
      }
    });
    dispatch(action);
  });
}

function useDispatchAsync(actionFunction, deps) {
  if (deps === void 0) {
    deps = [];
  }

  var dispatch = reactRedux.useDispatch(); // 👉 Better flow with informative & useful return

  var _useState = react.useState(undefined),
      result = _useState[0],
      setResult = _useState[1];

  var _useState2 = react.useState(undefined),
      error = _useState2[0],
      setError = _useState2[1];

  react.useEffect(function () {
    dispatchAsync(dispatch, actionFunction.apply(void 0, deps)).then(function (res) {
      return res.success ? setResult(res.result) : setError(res.error);
    })["catch"](function (e) {
      console.error('useDispatchAsync: Unexpected error', e);
      setError(e);
    });
  }, deps);
  var status = react.useMemo(function () {
    return !result && !error ? 'loading' : result ? 'success' : 'error';
  }, [result, error]);
  return {
    status: status,
    result: result,
    error: error
  };
}

exports.ConfigMiddleware = ConfigMiddleware;
exports.createDispatchAsyncMiddleware = createDispatchAsyncMiddleware;
exports.dispatchAsync = dispatchAsync;
exports.useDispatchAsync = useDispatchAsync;
//# sourceMappingURL=react-redux-dispatch-async.cjs.development.js.map
